@@ -38,9 +38,9 @@
 from abc import abstractmethod
 from typing import Optional
 
-from dimp import NetworkID, ID, Address
+from dimp import NetworkType, ID
 from dimp import User, Group
-from dimp import Meta, Profile
+from dimp import Meta, Document
 from dimp import Barrack
 
 from .network import ServiceProvider, Station, Robot
@@ -49,29 +49,65 @@ from .group import Polylogue
 
 class Facebook(Barrack):
 
-    #
-    #   Meta
-    #
-    @staticmethod
-    def verify_meta(meta: Meta, identifier: ID) -> bool:
-        assert meta is not None, 'meta should not be empty'
-        return meta.match_identifier(identifier)
+    @property
+    def current_user(self) -> Optional[User]:
+        """
+        Get current user (for signing and sending message)
+
+        :return: User
+        """
+        users = self.local_users
+        if users is not None and len(users) > 0:
+            return users[0]
 
     @abstractmethod
     def save_meta(self, meta: Meta, identifier: ID) -> bool:
-        """ Save meta into local storage """
+        """
+        Save meta for entity ID (must verify first)
+
+        :param meta:       entity meta
+        :param identifier: entity ID
+        :return: True on success
+        """
+        raise NotImplemented
+
+    @abstractmethod
+    def save_document(self, document: Document) -> bool:
+        """
+        Save entity document with ID (must verify first)
+
+        :param document: entity document
+        :return: True on success
+        """
         raise NotImplemented
 
     #
-    #   Profile
+    #   Group Members
     #
-    def verify_profile(self, profile: Profile, identifier: ID=None) -> bool:
-        assert profile is not None, 'profile should not be empty'
+    @abstractmethod
+    def save_members(self, members: list, identifier: ID) -> bool:
+        """
+        Save members of group
+
+        :param members:    member ID list
+        :param identifier: group ID
+        :return: True on success
+        """
+        raise NotImplemented
+
+    #
+    #   Document checking
+    #
+    @staticmethod
+    def is_empty_document(document: Document) -> bool:
+        if document is None:
+            return True
+        json = document.get('data')
+        return json is None or len(json) == 0
+
+    def is_valid_document(self, document: Document) -> bool:
+        identifier = document.identifier
         if identifier is None:
-            identifier = self.identifier(profile.identifier)
-            assert identifier is not None, 'profile error: %s ' % profile
-        elif identifier != profile.identifier:
-            # profile ID not match
             return False
         # NOTICE: if this is a group profile,
         #             verify it with each member's meta.key
@@ -86,14 +122,14 @@ class Facebook(Barrack):
                     if meta is None:
                         # FIXME: meta not found for this member
                         continue
-                    if profile.verify(public_key=meta.key):
+                    if document.verify(public_key=meta.key):
                         return True
             # DISCUSS: what to do about assistants?
 
             # check by owner
             owner = self.owner(identifier=identifier)
             if owner is None:
-                if identifier.type == NetworkID.Polylogue:
+                if identifier.type == NetworkType.POLYLOGUE:
                     # NOTICE: if this is a polylogue profile
                     #             verify it with the founder's meta.key
                     #             (which equals to the group's meta.key)
@@ -107,80 +143,28 @@ class Facebook(Barrack):
             else:
                 meta = self.meta(identifier=owner)
         else:
-            assert identifier.is_user, 'profile ID error: %s' % identifier
             meta = self.meta(identifier=identifier)
         if meta is not None:
-            return profile.verify(public_key=meta.key)
-
-    @abstractmethod
-    def save_profile(self, profile: Profile, identifier: ID=None) -> bool:
-        """ Save profile into database """
-        raise NotImplemented
-
-    #
-    #   Group Members
-    #
-    @abstractmethod
-    def save_members(self, members: list, identifier: ID) -> bool:
-        """ Save members into database """
-        raise NotImplemented
-
-    #
-    #   All local users (for decrypting received message)
-    #
-    @property
-    def local_users(self) -> Optional[list]:
-        raise NotImplemented
-
-    #
-    #   Current user (for signing and sending message)
-    #
-    @property
-    def current_user(self) -> Optional[User]:
-        users = self.local_users
-        if users is not None and len(users) > 0:
-            return users[0]
-
-    def __identifier(self, address: Address) -> Optional[ID]:
-        """ generate ID from meta with address """
-        identifier = ID.new(address=address)
-        meta = self.meta(identifier=identifier)
-        if meta is None:
-            # failed to get meta for this address
-            return None
-        seed = meta.seed
-        if seed is None or len(seed) == 0:
-            return identifier
-        identifier = meta.generate_identifier(address.network)
-        self.cache_id(identifier)
-        return identifier
-
-    def create_identifier(self, string: str) -> ID:
-        if isinstance(string, Address):
-            # convert Address to ID
-            return self.__identifier(address=string)
-        assert isinstance(string, str), 'ID error: %s' % string
-        return ID(string)
+            return document.verify(public_key=meta.key)
 
     def create_user(self, identifier: ID) -> User:
-        assert identifier.is_user, 'user ID error: %s' % identifier
         if identifier.is_broadcast:
             # create user 'anyone@anywhere'
             return User(identifier=identifier)
         # make sure meta exists
         assert self.meta(identifier) is not None, 'failed to get meta for user: %s' % identifier
+        # TODO: make sure visa key exists before calling this
         # check user type
         u_type = identifier.type
-        if u_type == NetworkID.Main or u_type == NetworkID.BTCMain:
+        if u_type in [NetworkType.MAIN, NetworkType.BTC_MAIN]:
             return User(identifier=identifier)
-        if u_type == NetworkID.Robot:
+        if u_type == NetworkType.ROBOT:
             return Robot(identifier=identifier)
-        if u_type == NetworkID.Station:
+        if u_type == NetworkType.STATION:
             return Station(identifier=identifier)
         raise TypeError('unsupported user type: %s' % u_type)
 
     def create_group(self, identifier: ID) -> Group:
-        assert identifier.is_group, 'group ID error: %s' % identifier
         if identifier.is_broadcast:
             # create group 'everyone@everywhere'
             return Group(identifier=identifier)
@@ -188,78 +172,10 @@ class Facebook(Barrack):
         assert self.meta(identifier) is not None, 'failed to get meta for group: %s' % identifier
         # check group type
         g_type = identifier.type
-        if g_type == NetworkID.Polylogue:
+        if g_type == NetworkType.POLYLOGUE:
             return Polylogue(identifier=identifier)
-        if g_type == NetworkID.Chatroom:
+        if g_type == NetworkType.CHATROOM:
             raise NotImplementedError('Chatroom not implemented')
-        if g_type == NetworkID.Provider:
+        if g_type == NetworkType.PROVIDER:
             return ServiceProvider(identifier=identifier)
         raise TypeError('unsupported group type: %s' % g_type)
-
-    #
-    #   GroupDataSource
-    #
-    def founder(self, identifier: ID) -> Optional[ID]:
-        uid = super().founder(identifier=identifier)
-        if uid is not None:
-            return uid
-        # check each member's public key with group meta
-        members = self.members(identifier=identifier)
-        if members is not None:
-            meta = self.meta(identifier=identifier)
-            if meta is not None:
-                # if the member's public key matches with the group's meta,
-                # it means this meta was generate by the member's private key
-                for item in members:
-                    m = self.meta(identifier=self.identifier(item))
-                    if m is not None and meta.match_public_key(m.key):
-                        # got it
-                        return item
-        # TODO: load founder from database
-
-    def owner(self, identifier: ID) -> ID:
-        uid = super().owner(identifier=identifier)
-        if uid is not None:
-            return uid
-        # check group type
-        if identifier.type == NetworkID.Polylogue:
-            # Polylogue's owner is its founder
-            return self.founder(identifier=identifier)
-        # TODO: load owner from database
-
-    def is_founder(self, member: ID, group: ID) -> bool:
-        # check member's public key with group's meta.key
-        g_meta = self.meta(identifier=group)
-        if g_meta is None:
-            raise LookupError('failed to get meta for group: %s' % group)
-        m_meta = self.meta(identifier=member)
-        if m_meta is None:
-            raise LookupError('failed to get meta for member: %s' % member)
-        return g_meta.match_public_key(m_meta.key)
-
-    def is_owner(self, member: ID, group: ID) -> bool:
-        if group.type == NetworkID.Polylogue:
-            return self.is_founder(member=member, group=group)
-        else:
-            raise NotImplementedError('only Polylogue so far')
-
-    def exists_member(self, member: ID, group: ID) -> bool:
-        members = self.members(identifier=group)
-        if members is not None and member in members:
-            return True
-        owner = self.owner(identifier=group)
-        if member == owner:
-            return True
-
-    #
-    #   Group Assistants
-    #
-    @abstractmethod
-    def assistants(self, identifier: ID) -> Optional[list]:
-        """ Get assistants for this group """
-        raise NotImplemented
-
-    def exists_assistant(self, member: ID, group: ID) -> bool:
-        assistants = self.assistants(identifier=group)
-        if assistants is not None:
-            return member in assistants
